@@ -1,75 +1,69 @@
+#![cfg_attr(target_family = "wasm", feature(wasi_ext))]
+
 mod file_ext;
 mod page;
 mod page_no;
 mod size_info;
 
-pub use page::Page;
+pub use page::*;
 pub use page_no::*;
 
 use file_ext::FileExt;
 use size_info::SizeInfo;
+use std::convert::TryInto;
 use std::fs::File;
 use std::io::{Error, ErrorKind, Result};
-use std::path::Path;
 use std::sync::atomic::{AtomicU32, Ordering};
 
-use page_lock::PageLocker;
+use page_lock::RwLock;
 use tokio::task::spawn_blocking;
 
 /// **NOTE**: Caller must ensure that closure must not panic.
 /// If it does, it will be undefined behavior.
-macro_rules! into_async { [$body:expr] => { unsafe { spawn_blocking(move || $body).await.unwrap_unchecked() } }; }
-macro_rules! err { [$cond: expr, $msg: expr] => { if $cond { return Err(Error::new(ErrorKind::InvalidInput, $msg)); } }; }
+macro_rules! into_async { [$body: expr] => { unsafe { spawn_blocking(move || $body).await.unwrap_unchecked() } }; }
 
 pub struct Pages<N, const SIZE: usize> {
     /// Total Page number
     len: AtomicU32,
+    rwlock: RwLock<N>,
     file: &'static File,
-    locker: PageLocker<N>,
-    size_info: SizeInfo,
+    metadata: [u8; SIZE],
 }
 
 impl<N: PageNo, const SIZE: usize> Pages<N, SIZE> {
     /// Create a new `Pages` instance.
-    pub fn open(path: impl AsRef<Path>) -> Result<Self> {
-        err!(SIZE < 64, "Page size should >= 64 bytes");
-        err!(SIZE > 1024 * 256, "Page size should > 256 kilobytes");
-
+    pub fn open(file: File) -> Result<Self> {
         let size_info = SizeInfo {
             block_size: SIZE as u32,
             pages_len_nbytes: N::SIZE as u8,
         };
-        let file = File::options()
-            .read(true)
-            .write(true)
-            .create(true)
-            .open(path)?;
-
         let file_len = file.metadata()?.len();
         // So that, There is no residue bytes.
         if file_len % SIZE as u64 != 0 {
             return Err(ErrorKind::InvalidData.into());
         }
+        let mut metadata = [0; SIZE];
         let len = if file_len == 0 {
             // Is new file? set the length to 1.
             file.set_len(SIZE as u64)?;
             1
         } else {
-            // Exist File?  Check size info.
-            let mut buf = [0; 4];
-            file.read_exact_at(&mut buf, 0)?;
-            let info = SizeInfo::from(buf);
-            err!(
-                info != size_info,
-                format!("Expected {:?}, but got: {:?}", info, size_info)
-            );
+            file.read_exact_at(&mut metadata, 0)?;
+            let info = SizeInfo::from(metadata[0..4].try_into().unwrap());
+            if info != size_info {
+                return Err(Error::new(
+                    ErrorKind::InvalidInput,
+                    format!("Expected {:?}, but got: {:?}", info, size_info),
+                ));
+            }
+            metadata[0..4].copy_from_slice(&size_info.to_bytes());
             file_len as u32 / SIZE as u32
         };
         Ok(Self {
-            size_info,
             len: AtomicU32::new(len),
             file: Box::leak(Box::new(file)),
-            locker: PageLocker::new(),
+            rwlock: RwLock::new(),
+            metadata,
         })
     }
 
@@ -83,9 +77,9 @@ impl<N: PageNo, const SIZE: usize> Pages<N, SIZE> {
         into_async!(file.sync_all())
     }
 
-    pub async fn read(&self, num: N) -> Result<[u8; SIZE]> {
+    /// Read a page, without locking.
+    pub async fn get(&self, num: N) -> Result<[u8; SIZE]> {
         debug_assert!(num.as_u32() < self.len());
-        self.locker.unlock(num).await;
         let num = num.as_u32() as u64;
         let file = self.file;
         into_async!({
@@ -95,9 +89,15 @@ impl<N: PageNo, const SIZE: usize> Pages<N, SIZE> {
         })
     }
 
+    pub async fn read(&self, num: N) -> Result<Reader<'_, N, [u8; SIZE]>> {
+        let _guard = self.rwlock.read(num).await;
+        let data = self.get(num).await?;
+        Ok(Reader { data, _guard })
+    }
+
     pub async fn goto(&self, num: N) -> Result<Page<'_, N, SIZE>> {
-        let buf = self.read(num).await?;
-        let _lock = self.locker.lock(num);
+        let _lock = self.rwlock.write(num).await;
+        let buf = self.get(num).await?;
         Ok(Page {
             _lock,
             num,
@@ -133,70 +133,25 @@ impl<N: PageNo, const SIZE: usize> Pages<N, SIZE> {
         Ok(N::new(old_len))
     }
 
+    #[inline]
     #[allow(clippy::len_without_is_empty)]
     pub fn len(&self) -> u32 {
-        self.len.load(Ordering::SeqCst)
+        self.len.load(Ordering::Relaxed)
     }
 
+    #[inline]
     pub fn inner(&self) -> &File {
         self.file
     }
+
+    /// Get a mutable reference to the pages's metadata.
+    pub fn metadata(&self) -> *mut [u8] {
+        &self.metadata[4..] as *const _ as *mut _
+    }
 }
 
 impl<N, const SIZE: usize> Drop for Pages<N, SIZE> {
     fn drop(&mut self) {
-        self.file
-            .write_all_at(&self.size_info.to_bytes(), 0)
-            .unwrap();
-    }
-}
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    async fn read_write() -> Result<()> {
-        let pages = Pages::<u16, 64>::open("test1.db")?;
-
-        assert_eq!(pages.len(), 1);
-        pages.create([0; 64]).await?;
-        assert_eq!(pages.len(), 2);
-
-        let mut page = pages.goto(1).await?;
-        page.buf = [1; 64];
-        page.write().await?;
-
-        assert_eq!(pages.read(1).await?, [1; 64]);
-        pages.alloc(1).await?;
-
-        assert_eq!(pages.len(), 3);
-
-        pages.write(2, [2; 64]).await?;
-        assert_eq!(pages.read(2).await?, [2; 64]);
-
-        std::fs::remove_file("test1.db")
-    }
-
-    async fn alloc_page() -> Result<()> {
-        let pages = Pages::<u16, 64>::open("test2.db")?;
-
-        pages.create([1; 64]).await?;
-        assert_eq!(pages.len(), 2);
-        pages.alloc(1).await?;
-        assert_eq!(pages.len(), 3);
-
-        std::fs::remove_file("test2.db")
-    }
-
-    #[test]
-    fn all() {
-        tokio::runtime::Builder::new_current_thread()
-            .enable_all()
-            .build()
-            .expect("Failed building the Runtime")
-            .block_on(async {
-                read_write().await.unwrap();
-                alloc_page().await.unwrap();
-            });
+        self.file.write_all_at(&self.metadata, 0).unwrap();
     }
 }
